#!/usr/bin/env -S node --loader ts-node/esm
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tstest_1 = require("tstest");
const state_switch_js_1 = require("./state-switch.js");
tstest_1.test('on()', async (t) => {
    const ss = new state_switch_js_1.StateSwitch();
    t.notOk(ss.on(), 'default is not on');
    ss.on('pending');
    t.equal(ss.on(), 'pending', 'should be state pending');
    ss.on(true);
    t.equal(ss.on(), true, 'should be state true');
    t.notOk(ss.off(), 'should not off');
    ss.off(true);
    t.notOk(ss.on(), 'should not ON after off()');
});
tstest_1.test('off()', async (t) => {
    const ss = new state_switch_js_1.StateSwitch();
    t.ok(ss.off(), 'default is off');
    t.equal(ss.off(), true, 'should in state true');
    ss.off('pending');
    t.equal(ss.off(), 'pending', 'should be state pending');
    ss.off(true);
    t.equal(ss.off(), true, 'should be state true');
    t.notOk(ss.on(), 'should not on');
    ss.on(true);
    t.notOk(ss.off(), 'should not OFF after on()');
});
tstest_1.test('pending', async (t) => {
    const ss = new state_switch_js_1.StateSwitch();
    t.notOk(ss.pending, 'default is not pending');
    ss.on('pending');
    t.ok(ss.pending, 'should in pending state');
    ss.on(true);
    t.notOk(ss.pending, 'should not in pending state');
    ss.off('pending');
    t.ok(ss.pending, 'should in pending state');
});
tstest_1.test('name', async (t) => {
    const CLIENT_NAME = 'StateSwitchTest';
    const ss = new state_switch_js_1.StateSwitch(CLIENT_NAME);
    t.equal(ss.name, CLIENT_NAME, 'should get the same client name as init');
});
tstest_1.test('version()', t => {
    const ss = new state_switch_js_1.StateSwitch();
    t.ok(ss.version(), 'should get version');
    t.end();
});
tstest_1.test('ready()', async (t) => {
    const spy = tstest_1.sinon.spy();
    const ss = new state_switch_js_1.StateSwitch();
    ss.ready('off').then(() => spy('off')).catch(() => t.fail('rejection'));
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should be read off at the initial state');
    spy.resetHistory();
    ss.ready('on', true).catch(() => spy('on'));
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should catch the exception when noCross=true');
    spy.resetHistory();
    ss.ready('on').then(() => spy('on')).catch(() => t.fail('rejection'));
    ss.on(true);
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should ready(on)');
    spy.resetHistory();
    ss.ready('on').then(() => spy('on')).catch(() => t.fail('rejection'));
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should ready(on) when already on');
    spy.resetHistory();
    ss.ready('off').then(() => spy('off')).catch(() => t.fail('rejection'));
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 0, 'should not ready(off) when its on');
    ss.off(true);
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should ready(off) after call off(true)');
});
tstest_1.test('ready() without default arg for waiting current state', async (t) => {
    const spy = tstest_1.sinon.spy();
    const ss = new state_switch_js_1.StateSwitch();
    ss.off('pending');
    ss.ready().then(() => spy('off')).catch(() => t.fail('rejection'));
    ss.off(true);
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should be ready() for off(true)');
    spy.resetHistory();
    ss.on('pending');
    ss.ready().then(() => spy('on')).catch(() => t.fail('rejection'));
    ss.on(true);
    await new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should be ready() for on(true)');
});
tstest_1.test('on/off events emitting', async (t) => {
    const spyOn = tstest_1.sinon.spy();
    const spyOff = tstest_1.sinon.spy();
    const ss = new state_switch_js_1.StateSwitch();
    ss.addListener('on', spyOn);
    ss.addListener('off', spyOff);
    t.ok(spyOn.notCalled, 'spyOn is not called');
    t.ok(spyOff.notCalled, 'spyOff is not called');
    ss.on('pending');
    t.ok(spyOn.calledOnce, 'spyOn is called once after on(pending)');
    t.same(spyOn.args[0], ['pending'], 'spyOn should be called with `pending` arg');
    t.ok(spyOff.notCalled, 'spyOff is not called');
    ss.on(true);
    t.ok(spyOn.calledTwice, 'spyOn is called once after on(pending)');
    t.same(spyOn.args[1], [true], 'spyOn should be called with `true` arg');
    t.ok(spyOff.notCalled, 'spyOff is not called');
    ss.off('pending');
    t.ok(spyOff.calledOnce, 'spyOff is called once after off(pending)');
    t.same(spyOff.args[0], ['pending'], 'spyOff should be called with `pending` arg');
    ss.off(true);
    t.ok(spyOff.calledTwice, 'spyOff is called twice after off(true)');
    t.same(spyOff.args[1], [true], 'spyOff should be called with `true` arg');
    t.ok(spyOn.calledTwice, 'spyOn called twice at last');
});
//# sourceMappingURL=state-switch.spec.js.map